@@ -73,23 +73,23 @@ void SmsSender::printInfo(){
 void SmsSender::printUsage()
 {
     consoleOutput("Usage:");
-    consoleOutput("  smssend [options] -n +39xxxxxx:+39xxxxxx -m \"Text\"");
+    consoleOutput("  smssend [options] -n +39xxxxxx[:+39xxxxxx[...]] -m \"Text\"");
     consoleOutput("Options:");
     consoleOutput("  -c         : compose the message with conversation");
-    consoleOutput("  -C         : send the message with conversation");
+    consoleOutput("  -C         : compose the message with conversation and send");
     consoleOutput("");
-    consoleOutput("Following options are valid only if used without -c or -C");
+    consoleOutput("The following options are valid only if used without -c or -C");
     consoleOutput("  -l logfile : log pnatd messages to logfile");
     consoleOutput("  -s         : don't save message in the sent folder");
-    consoleOutput("  -f         : send a flash sms");
+    consoleOutput("  -f         : send a flash SMS");
     //consoleOutput("  -r         : request delivery report");
-    consoleOutput("  -v VP      : set the sms validity period (0-255)");
+    consoleOutput("  -v VP      : set the SMS validity period (0-255)");
     consoleOutput("               Default value is 167 (1 day)");
-    consoleOutput("                 0 - 143 (VP + 1) x 5 mins");
-    consoleOutput("               144 - 167 12 Hours + ((VP-143) x 30 mins)");
-    consoleOutput("               168 - 196 (VP-166) x 1 day");
-    consoleOutput("               197 - 255 (VP-192) x 1 week");
-    consoleOutput("  -p PIN     : pin code (for pin locked SIM)");
+    consoleOutput("                 0 - 143: (VP + 1) x 5 mins");
+    consoleOutput("               144 - 167: 12 Hours + ((VP-143) x 30 mins)");
+    consoleOutput("               168 - 196: (VP-166) x 1 day");
+    consoleOutput("               197 - 255: (VP-192) x 1 week");
+    consoleOutput("  -p PIN     : PIN code (for PIN locked SIM)");
 }
 
 int SmsSender::exec()
@@ -207,16 +207,16 @@ int SmsSender::exec()
             int parts = message.length() / 160;
             if (message.length() % 160 > 0)
                 parts++;
-            consoleOutput("WARNING: SMS exceding 160 characters");
-            consoleOutput(QString("         Splitted in %1 parts").arg(parts));
+            consoleOutput("WARNING: SMS exceeds 160 characters");
+            consoleOutput(QString("         Splitting into %1 parts").arg(parts));
             for (int i=0; i<parts; i++)
                 messageBlocks.append(message.mid(160*i, 160));
         }
 
         if (m_Flash)
-            consoleOutput("Sendind flash SMS:");
+            consoleOutput("Sending flash SMS:");
         else
-            consoleOutput("Sendind SMS:");
+            consoleOutput("Sending SMS:");
 
         bool sent;
         int totalMsg = numbers.count() * messageBlocks.count();
@@ -238,7 +238,7 @@ int SmsSender::exec()
             //Save the full message in the sent folders
             if (msgSent && !m_Compose && m_Save){
                 if (!saveMsgToSentFolder(number, messageBlocks.join("")))
-                    consoleOutput("WARNING: Failed to save sent sms");
+                    consoleOutput("WARNING: Failed to save sent SMS");
             }
         }
     }
@@ -259,7 +259,7 @@ bool SmsSender::compose(QString number, QString messageText)
     if (m_Send){
         if (service.send(message)){
             consoleOutput(".");
-            consoleOutput(QString("OK: sms queued for sending to %1").arg(number));
+            consoleOutput(QString("OK: SMS queued to be sent to %1").arg(number));
             return true;
         }
     }else{
@@ -268,7 +268,7 @@ bool SmsSender::compose(QString number, QString messageText)
             return true;
         }
     }
-    consoleOutput(QString("ERR: Failed to queue sms for sending to %1").arg(number));
+    consoleOutput(QString("ERR: Failed to queue SMS to be sent to %1").arg(number));
     return false;
 }
 
@@ -306,14 +306,22 @@ bool SmsSender::send(QString number, QString messageText)
             consoleOutput(".", false);
     }
 
+    if (res){
+        if (!writeCmd(pty, "AT+CSCS=\"HEX\"\r", "OK")){
+            consoleOutput(QString("\nERR: Failed to send at+cscs command"));
+            res = false;
+        }else
+            consoleOutput(".", false);
+    }
+
     if (res){
         int opt = 17;
         /*if (m_DeliveryReport)
             opt += 32;*/
 
-        int dcs = 0;
+        int dcs = 8;
         if (m_Flash)
-            dcs = 16;
+            dcs = 24;
 
         if (!writeCmd(pty, QString("AT+CSMP=%1,%2,0,%3\r").arg(opt).arg(m_Validity).arg(dcs), "OK")){
             consoleOutput(QString("\nERR: Failed to send at+csmp command"));
@@ -331,7 +339,15 @@ bool SmsSender::send(QString number, QString messageText)
     }
 
     if (res){
-        if (!writeCmd(pty, messageText, messageText.trimmed(), 1000)){
+        QString hex;
+        const QChar* u = messageText.unicode();
+        for (int i=0; i<messageText.length(); i++){
+            int val = u[i].unicode();
+            QString s = QString::number(val, 16);
+            hex.append(s.rightJustified(4, '0'));
+        }
+
+        if (!writeCmd(pty, hex, hex.trimmed(), 1000)){
             consoleOutput(QString("\nERR: Failed to send text message"));
             res = false;
         }else
@@ -346,7 +362,7 @@ bool SmsSender::send(QString number, QString messageText)
             consoleOutput("OK");
             /*if (m_Save){
                 if (!saveMsgToSentFolder(number, messageText))
-                    consoleOutput("WARNING: Failed to save sent sms");
+                    consoleOutput("WARNING: Failed to save sent SMS");
             }*/
         }
     }
@@ -381,31 +397,38 @@ bool SmsSender::writeCmd(Pty* p, QString cmd, QString okReply, int msectimeout)
 
 bool SmsSender::saveMsgToSentFolder(QString number, QString message)
 {
+    if (m_El == NULL)
+        return false;
     bool result = false;
     QString partNumber = number.right(7);
 
     RTComElEvent* ev = rtcom_el_event_new();
 
-    RTCOM_EL_EVENT_SET_FIELD(ev, service, g_strdup(QString("RTCOM_EL_SERVICE_SMS").toLatin1().data()));
-    RTCOM_EL_EVENT_SET_FIELD(ev, event_type, g_strdup(QString("RTCOM_EL_EVENTTYPE_SMS_OUTBOUND").toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, service, (gchar*)"RTCOM_EL_SERVICE_SMS");
+    RTCOM_EL_EVENT_SET_FIELD(ev, event_type, (gchar*)"RTCOM_EL_EVENTTYPE_SMS_OUTBOUND");
     RTCOM_EL_EVENT_SET_FIELD(ev, start_time, time(NULL));
-    RTCOM_EL_EVENT_SET_FIELD(ev, end_time, time(NULL));
+    RTCOM_EL_EVENT_SET_FIELD(ev, end_time, 0);
     RTCOM_EL_EVENT_SET_FIELD(ev, is_read, TRUE);
-    RTCOM_EL_EVENT_SET_FIELD(ev, remote_ebook_uid, 0);
-    RTCOM_EL_EVENT_SET_FIELD(ev, local_uid, g_strdup(QString("ring/tel/ring").toLatin1().data()));
-    RTCOM_EL_EVENT_SET_FIELD(ev, local_name, g_strdup(QString("<SelfHandle>").toLatin1().data()));
-    RTCOM_EL_EVENT_SET_FIELD(ev, remote_uid, number.toLatin1().data());
-    RTCOM_EL_EVENT_SET_FIELD(ev, remote_name, g_strdup(QString("").toLatin1().data()));
-    RTCOM_EL_EVENT_SET_FIELD(ev, group_uid, g_strdup(partNumber.toLatin1().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_ebook_uid, (gchar *)"0");
+    RTCOM_EL_EVENT_SET_FIELD(ev, local_uid, (gchar*)"ring/tel/ring");
+    RTCOM_EL_EVENT_SET_FIELD(ev, local_name, (gchar*)"<SelfHandle>");
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_uid, g_strdup(number.toAscii().data()));
+    RTCOM_EL_EVENT_SET_FIELD(ev, remote_name, (gchar*)"");
+    RTCOM_EL_EVENT_SET_FIELD(ev, group_uid, g_strdup(partNumber.toAscii().data()));
     RTCOM_EL_EVENT_SET_FIELD(ev, free_text, g_strdup(message.toUtf8().data()));
 
-    int id = rtcom_el_add_event(m_El, ev, NULL);
-    if (id >= 0){
-        QUuid uid = QUuid::createUuid();
-        rtcom_el_add_header(m_El, id, QString("message-token").toLatin1().data(), uid.toString().toLatin1().data(), NULL);
-        result = true;
+    GError *err = NULL;
+    for (int i=0; i<3; i++){
+        int id = rtcom_el_add_event(m_El, ev, &err);
+        if (id >= 0){
+            QUuid uid = QUuid::createUuid();
+            rtcom_el_add_header(m_El, id, (gchar*)"message-token", g_strdup(uid.toString().toAscii().data()), NULL);
+            //rtcom_el_fire_event_updated(m_El, id);
+            result = true;
+            break;
+        }
+        usleep(100000);
     }
-
     rtcom_el_event_free(ev);
 
     return result;
